package com.sztzjy.trade.controller.stu;



import com.sztzjy.trade.annotation.AnonymousAccess;
import com.sztzjy.trade.annotation.OperateLog;
import com.sztzjy.trade.config.exception.UnAuthorizedException;
import com.sztzjy.trade.config.security.JwtUser;
import com.sztzjy.trade.config.security.LoginResult;
import com.sztzjy.trade.config.security.TokenProvider;
import com.sztzjy.trade.entity.StuUser;
import com.sztzjy.trade.entity.StuUserExample;
import com.sztzjy.trade.entity.ZYUserInfo;
import com.sztzjy.trade.mapper.StuUserMapper;
import com.sztzjy.trade.service.StuUserService;
import com.sztzjy.trade.util.ResultDataEntity;
import com.sztzjy.trade.util.RsaUtil;
import com.sztzjy.trade.util.TzApi;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Api(tags = "用户管理")
@RequestMapping("api/user")
@RestController
public class UserController {


    @Resource
    private StuUserService stuUserService;
    @Resource
    private HttpServletRequest request;
    @Autowired
    private StuUserMapper userMapper;
    @Resource
    private AuthenticationManagerBuilder authenticationManagerBuilder;

    @AnonymousAccess
    @ApiOperation("用户登录")
    @PostMapping("login")
    @OperateLog(description = "登录授权")
    public ResultDataEntity<LoginResult> login(@ApiParam("用户名") String username,
                                               @ApiParam("加密后的密码") String passwordEncode,
                                               @ApiParam("智云携带的token")@RequestParam(required = false) String TOKEN,
                                               @RequestParam(required = false) Integer type) {
        JwtUser jwtUser;
        String password;
        if(org.apache.commons.lang3.StringUtils.isBlank(TOKEN)){ //子系统登录
            try {
                password = RsaUtil.decryptByPrivateKey(passwordEncode);
            } catch (Exception e) {
                return new ResultDataEntity(HttpStatus.BAD_REQUEST, "密码错误");
            }
            try {
                StuUserExample example = new StuUserExample();
                StuUserExample.Criteria criteria = example.createCriteria();
                criteria.andStudentIdEqualTo(username);
                List<StuUser> stuUsers = userMapper.selectByExample(example);
                if(stuUsers.isEmpty()){
                    throw new UnAuthorizedException("账号不存在");
                }else {
                    StuUser stuUser = stuUsers.get(0);
                    LoginResult loginResult = new LoginResult();
                    if(org.apache.commons.lang3.StringUtils.isBlank(stuUser.getPassword()) && "123qwe".equals(password)){
                        stuUser.setPassword("123qwe");
                        if(org.apache.commons.lang3.StringUtils.isBlank(stuUser.getClassId())){
                            stuUser.setRoleId(3);
                        }else {
                            stuUser.setRoleId(4);
                        }
                        userMapper.updateByPrimaryKeySelective(stuUser);
                        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(username, password);
                        Authentication authentication = authenticationManagerBuilder.getObject().authenticate(authenticationToken);
                        SecurityContextHolder.getContext().setAuthentication(authentication);
                        jwtUser = (JwtUser) authentication.getPrincipal();
                        String token = TokenProvider.createToken(jwtUser);

                        loginResult.setUserId(stuUser.getUserId());
                        loginResult.setUsername(stuUser.getSchoolId());
                        loginResult.setRoleId(stuUser.getRoleId());
                        loginResult.setSchoolId(Integer.valueOf(stuUser.getSchoolId()));
                        loginResult.setAccessToken(token);
                        return new ResultDataEntity<>(HttpStatus.OK,loginResult);
                    }else {
                        if(password.equals(stuUser.getPassword())){
                            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(username, password);
                            Authentication authentication = authenticationManagerBuilder.getObject().authenticate(authenticationToken);
                            SecurityContextHolder.getContext().setAuthentication(authentication);
                            jwtUser = (JwtUser) authentication.getPrincipal();
                            String token = TokenProvider.createToken(jwtUser);

                            loginResult.setUserId(stuUser.getUserId());
                            loginResult.setUsername(stuUser.getSchoolId());
                            loginResult.setRoleId(stuUser.getRoleId());
                            loginResult.setSchoolId(Integer.valueOf(stuUser.getSchoolId()));
                            loginResult.setAccessToken(token);
                            return new ResultDataEntity<>(HttpStatus.OK,loginResult);
                        }else {
                            throw new UnAuthorizedException("密码错误");
                        }
                    }
                }
            } catch (Exception e) {
                throw new UnAuthorizedException("密码错误");
            }
        }else {  //单点登录
            if (!StringUtils.hasText(username) && !StringUtils.hasText(passwordEncode) && !StringUtils.hasText(TOKEN)) {
                throw new IllegalArgumentException("请提供登录凭据");
            }
            if (StringUtils.hasText(TOKEN)) {

                if (type==null)
                {
                    type = 0;
                }

                if (type ==1 )
                {
                    jwtUser = TokenProvider.getJWTUserOne(TOKEN);
                }else {
                    jwtUser = TokenProvider.getJWTUserByZhiYun(TOKEN);
                }

              //  jwtUser = TokenProvider.getJWTUserByZhiYun(TOKEN);



                if (jwtUser == null) {
                    throw new UnAuthorizedException("token无效");
                }
            } else {
                try {
                    password = RsaUtil.decryptByPrivateKey(passwordEncode);
                } catch (Exception e) {
                    throw new UnAuthorizedException("密码错误");
                }

                jwtUser = TzApi.foreignExchangeTradingLogin(username, password);
                if (jwtUser == null) {
                    throw new UnAuthorizedException("用户名或密码错误");
                }
            }
            Authentication authentication = new UsernamePasswordAuthenticationToken(jwtUser, null, jwtUser.getAuthorities());
            SecurityContextHolder.getContext().setAuthentication(authentication);
            String token = TokenProvider.createToken(jwtUser);

//        JwtUser user = TokenProvider.getJWTUser(request);
//        stuUserService.insertAll(jwtUser);
            return new ResultDataEntity<LoginResult>(HttpStatus.OK,LoginResult.create(jwtUser, token));
        }

    }



    @ApiOperation("明文密码公钥加密方法（该方法仅在开发阶段开放用于测试接口）")
    @AnonymousAccess
    @GetMapping("encrypt")
    public ResultDataEntity<String> encrypt(@ApiParam("明文密码") String pwd) throws Exception {
        return new ResultDataEntity<>(HttpStatus.OK,"加密成功",RsaUtil.encryptByPublicKey(pwd));
    }



    @ApiOperation("同步学生信息")
    @PostMapping("synchronizationStudentInfo")
    public ResultDataEntity synchronizationStudentInfo() {
        JwtUser user = TokenProvider.getJWTUser(request);
        stuUserService.insertAll(user);
        return new ResultDataEntity(HttpStatus.OK);
    }


    @ApiOperation("开放接口,自动同步学生信息")
    @PostMapping("synchronizationStudentInfoByAuto")
    @AnonymousAccess
    public ResultDataEntity synchronizationStudentInfoByAuto(@RequestBody List<ZYUserInfo> studentList) {
//        JwtUser user = TokenProvider.getJWTUser(request);
        stuUserService.insertAllByAuto(studentList);
        return new ResultDataEntity(HttpStatus.OK);
    }


}
