package com.sztzjy.trade.controller.tch;

import com.sztzjy.trade.annotation.AnonymousAccess;
import com.sztzjy.trade.entity.dto.tch.TchHomeworkExamDTO;
import com.sztzjy.trade.entity.dto.tch.TchTopicTypeDTO;
import com.sztzjy.trade.service.TchHomeworkExamService;
import com.sztzjy.trade.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author tz
 * @date 2024/8/5 14:45
 */
@Api(tags = "作业考试页面管理")
@RequestMapping("api/tch/homeworkExam")
@RestController
public class TchHomeworkExamController {

    @Resource
    TchHomeworkExamService tchHomeworkExamService;

    @ApiOperation("题库作业/附件作业/考试展示")
    @PostMapping("/getHomework")
    @AnonymousAccess
    public ResultEntity getHomework(@ApiParam("用户ID") String userId,
                                    @ApiParam("作业类型(题库作业/附件作业/考试)") String type,
                                    @ApiParam("当前页") Integer index,
                                    @ApiParam("每页条数") Integer size) {

        return new ResultEntity<>(HttpStatus.OK,"成功",tchHomeworkExamService.getHomework(userId,type,index,size));

    }


    @ApiOperation("新增(作业/考试)")
    @PostMapping("/add")
    @AnonymousAccess
    public ResultEntity add(@RequestBody TchHomeworkExamDTO homeworkExamDTO,
                            @RequestParam(required = false)MultipartFile file) {

        Integer result=tchHomeworkExamService.add(homeworkExamDTO,file);

        if(result>0){
            return new ResultEntity<>(HttpStatus.OK,"新增成功");
        }else {
            return new ResultEntity<>(HttpStatus.OK,"新增失败");
        }
    }

    @ApiOperation("编辑(作业/考试)")
    @PostMapping("/update")
    @AnonymousAccess
    public ResultEntity update(@RequestBody TchHomeworkExamDTO homeworkExamDTO,
                               @RequestParam(required = false)MultipartFile file) {

        Integer result=tchHomeworkExamService.update(homeworkExamDTO,file);

        if(result>0){
            return new ResultEntity<>(HttpStatus.OK,"编辑成功");
        }else {
            return new ResultEntity<>(HttpStatus.OK,"编辑失败");
        }
    }


    @ApiOperation("编辑(根据ID回显)")
    @PostMapping("/getById")
    @AnonymousAccess
    public ResultEntity getById(@ApiParam("作业/考试ID") String id,
                                @ApiParam("作业类型(题库作业/附件作业/考试)") String type) {


        return new ResultEntity<>(HttpStatus.OK,"成功",tchHomeworkExamService.getById(id,type));
    }

    @ApiOperation("班级下拉框展示")
    @PostMapping("/getSelect")
    @AnonymousAccess
    public ResultEntity getSelect(String userId) {

        return new ResultEntity<>(HttpStatus.OK,"成功",tchHomeworkExamService.getSelect(userId));
    }

    @ApiOperation("随机抽题--展示题目类型和数量")
    @PostMapping("/getTopicType")
    @AnonymousAccess
    public ResultEntity getTopicType() {


        return new ResultEntity<>(HttpStatus.OK,"成功",tchHomeworkExamService.getTopicType());
    }
}
