package com.sztzjy.trade.controller.tch;

import cn.hutool.core.date.DateTime;
import com.sztzjy.trade.annotation.AnonymousAccess;
import com.sztzjy.trade.entity.TchStartCourseManage;
import com.sztzjy.trade.entity.dto.tch.TchAddStuDTO;
import com.sztzjy.trade.entity.dto.tch.TchStartCourseDTO;
import com.sztzjy.trade.entity.dto.tch.TchStartCourseNameListDTO;
import com.sztzjy.trade.service.TchTeachingManagementService;
import com.sztzjy.trade.util.ResultEntity;
import com.sztzjy.trade.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.List;

/**
 * @author tz
 * @date 2024/8/1 13:26
 */
@Api(tags = "教学管理页面")
@RequestMapping("api/tch/teachingManagement")
@RestController
public class TchTeachingManagementController {
    @Resource
    TchTeachingManagementService teachingManagementService;

    @Value("${file.path}")
    private String filePath;

    @Resource
    private IFileUtil IFileUtil;

    @ApiOperation("开课班级展示")
    @PostMapping("/getStartCourseList")
    @AnonymousAccess
    public ResultEntity getStartCourseList(@ApiParam("用户ID") String userId,
                                           @ApiParam("当前页") Integer index,
                                           @ApiParam("每页条数") Integer size) {

        return new ResultEntity<>(HttpStatus.OK,"成功",teachingManagementService.getStartCourseList(userId,index,size));

    }

    @ApiOperation("新增开课班级--下拉框查看")
    @PostMapping("/getSelect")
    @AnonymousAccess
    public ResultEntity getSelect(String userId,String schoolId) {

        return new ResultEntity<>(HttpStatus.OK,"成功",teachingManagementService.getSelect(userId,schoolId));

    }

    @ApiOperation("新增开课班级")
    @PostMapping("/addStartCourseList")
    @AnonymousAccess
    public ResultEntity addStartCourse(@RequestBody TchStartCourseDTO dto) {

        Integer result=teachingManagementService.addStartCourse(dto);

        if(result>0){
            return new ResultEntity<>(HttpStatus.OK,"新增成功");
        }else {
            return new ResultEntity<>(HttpStatus.OK,"新增失败");
        }

    }

    @ApiOperation("开课学生名单--查看")
    @PostMapping("/getStartCourseNameList")
    @AnonymousAccess
    public ResultEntity getStartCourseNameList(@ApiParam("开课ID") String startCourseId,
                                               @ApiParam("当前页") Integer index,
                                               @ApiParam("每页条数") Integer size) {

        return new ResultEntity<>(HttpStatus.OK,"成功",teachingManagementService.getStartCourseNameList(startCourseId,index,size));

    }

    @ApiOperation("开课学生名单--添加学生--回显")
    @PostMapping("/addStuEcho")
    @AnonymousAccess
    public ResultEntity addStuEcho(@ApiParam("开课ID") String startCourseId,
                                   @ApiParam("学校ID") String schoolId,
                                   @ApiParam("当前页") Integer index,
                                   @ApiParam("每页条数") Integer size) {

        return new ResultEntity<>(HttpStatus.OK,"成功",teachingManagementService.addStuEcho(startCourseId,schoolId,index,size));

    }


    @ApiOperation("开课学生名单--添加学生")
    @PostMapping("/addStu")
    @AnonymousAccess
    public ResultEntity addStu(@RequestBody TchAddStuDTO dto) {

        Integer result=teachingManagementService.addStu(dto);

        if(result>0){
            return new ResultEntity<>(HttpStatus.OK,"添加成功");
        }else {
            return new ResultEntity<>(HttpStatus.ACCEPTED,"添加失败，请联系管理处理");
        }

    }


    @ApiOperation("开课学生名单--添加班级")
    @PostMapping("/addClass")
    @AnonymousAccess
    public ResultEntity addClass(List<String> className,String schoolId,String startCourseId) {

        Integer result=teachingManagementService.addClass(className,schoolId,startCourseId);

        if(result>0){
            return new ResultEntity<>(HttpStatus.OK,"添加成功");
        }else {
            return new ResultEntity<>(HttpStatus.ACCEPTED,"添加失败，请联系管理处理");
        }

    }

    @ApiOperation("开课学生名单--批量导入")
    @PostMapping("/batchImport")
    @AnonymousAccess
    public ResultEntity batchImport(@RequestPart MultipartFile file,
                                    @ApiParam("开课ID")String startCourseId) {

        //验证文件类型
        if (!file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xls") && !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xlsx")) {
            return  new ResultEntity<>(HttpStatus.ACCEPTED, "文件类型有误！请上传Excel文件");
        }

        return new ResultEntity<>(HttpStatus.OK,"成功",teachingManagementService.batchImport(file,startCourseId));

    }


    @ApiOperation("Excel表格下载")
    @GetMapping("/excelDownload")
    @AnonymousAccess
    public void excelDownload(HttpServletResponse response) {

        String path=filePath;
        IFileUtil.download(response,path);
    }
}
