package com.sztzjy.trade.service.impl;

import com.github.pagehelper.PageInfo;
import com.sztzjy.trade.config.exception.handler.ServiceException;
import com.sztzjy.trade.entity.*;
import com.sztzjy.trade.entity.dto.tch.TchHomeworkExamDTO;
import com.sztzjy.trade.entity.dto.tch.TchTopicDetailsDTO;
import com.sztzjy.trade.entity.dto.tch.TchTopicTypeDTO;
import com.sztzjy.trade.mapper.*;
import com.sztzjy.trade.service.TchHomeworkExamService;
import com.sztzjy.trade.util.ConvertUtil;
import com.sztzjy.trade.util.PageUtil;
import com.sztzjy.trade.util.file.LocalFileUtil;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * @author tz
 * @date 2024/8/5 14:58
 */
@Service
public class TchHomeworkExamServiceImpl implements TchHomeworkExamService {
    @Resource
    TchHomeworkManageMapper homeworkManageMapper;

    @Resource
    ConvertUtil convertUtil;

    @Resource
    LocalFileUtil localFileUtil;

    @Resource
    TchExamManageMapper examManageMapper;
    @Resource
    TchStartCourseManageMapper startCourseManageMapper;
    @Resource
    StuLearningAssessmentMapper learningAssessmentMapper;
    @Resource
    TchTopicDetailsMapper topicDetailsMapper;
    @Override
    public PageInfo getHomework(String userId, String type, Integer index, Integer size) {
        if(!type.equals("考试")){
            TchHomeworkManageExample homeworkManageExample=new TchHomeworkManageExample();
            homeworkManageExample.createCriteria().andUserIdEqualTo(userId).andHomeworkTypeEqualTo(type);
            List<TchHomeworkManage> tchHomeworkManages = homeworkManageMapper.selectByExample(homeworkManageExample);


            return PageUtil.pageHelper(tchHomeworkManages, index, size);
        }else {
            TchExamManageExample examManageExample=new TchExamManageExample();
            examManageExample.createCriteria().andUserIdEqualTo(userId);
            List<TchExamManage> tchExamManages = examManageMapper.selectByExample(examManageExample);

            return PageUtil.pageHelper(tchExamManages,index,size);
        }

    }

    @Override
    public Integer add(TchHomeworkExamDTO homeworkExamDTO, MultipartFile file) {

        //将班级集合集合转为字符串
        String className="";
        for (int i = 0; i < homeworkExamDTO.getClassNameList().size(); i++) {

            if(homeworkExamDTO.getClassNameList().size()-1==i){
                className=className+homeworkExamDTO.getClassNameList().get(0);
            }else {
                className=className+homeworkExamDTO.getClassNameList().get(0)+",";
            }
        }

        // TODO: 2024/8/6 取出所选题的数量，在题库中随机抽取，将题目ID拼接成字符串存到数据库中
        //取出所选题的总数量
        List<TchTopicDetailsDTO> topicDetailsDTOS = homeworkExamDTO.getTopicDetailsDTOS();
        int quantity=0;
        for (TchTopicDetailsDTO topicDetailsDTO: topicDetailsDTOS) {
            quantity=quantity+topicDetailsDTO.getQuantity();
        }

        //查询题库所有题
        List<StuLearningAssessment> stuLearningAssessments = learningAssessmentMapper.selectByExample(null);

        if(quantity>stuLearningAssessments.size()){
            throw new ServiceException(HttpStatus.ACCEPTED,"要选择的题目数量不能大于题库的总数量");
        }

        // 打乱集合
        Collections.shuffle(stuLearningAssessments);

        // 获取前quantity个元素
        List<StuLearningAssessment> subList = stuLearningAssessments.subList(0, quantity);

        //遍历集合，取出id拼接成字符串
        String topicId="";
        for (int i = 0; i < subList.size(); i++) {
            if(subList.size()-1==i){
                topicId=topicId+subList.get(0).getTopicId();
            }else {
                topicId=topicId+subList.get(0).getTopicId()+",";
            }
        }

        // TODO: 2024/8/6 1、题库作业，2、附件作业，3、考试

        String string = UUID.randomUUID().toString();
        int insert=0; //用于判断作业或考试是否添加成功
        if(homeworkExamDTO.getType().equals("题库作业")){

            //题库作业信息赋值
            TchHomeworkManage homeworkManage=new TchHomeworkManage();
            homeworkManage.setHomeworkId(string);
            homeworkManage.setHomeworkName(homeworkExamDTO.getName());
            homeworkManage.setHomeworkType(homeworkExamDTO.getType());
            homeworkManage.setHomeworkClass(className);
            homeworkManage.setPassRate(BigDecimal.valueOf(0));
            homeworkManage.setStartingTime(homeworkExamDTO.getStartingTime());
            homeworkManage.setEndTime(homeworkExamDTO.getEndTime());
            homeworkManage.setState(0);
            homeworkManage.setUserId(homeworkExamDTO.getUserId());
            homeworkManage.setCreateTime(new Date());
            homeworkManage.setAnswersFrequency(homeworkExamDTO.getAnswersFrequency());
            homeworkManage.setTopicNumberState(homeworkExamDTO.getTopicNumberState());
            homeworkManage.setOptionNumberState(homeworkExamDTO.getOptionNumberState());
            homeworkManage.setParsingState(homeworkExamDTO.getParsingState());
            homeworkManage.setAnswerSate(homeworkExamDTO.getAnswerSate());
            homeworkManage.setTopicId(topicId);

            insert = homeworkManageMapper.insert(homeworkManage);


        }

        if(homeworkExamDTO.getType().equals("附件作业")){

            //附件作业信息赋值
            TchHomeworkManage homeworkManage=new TchHomeworkManage();
            homeworkManage.setHomeworkId(string);
            homeworkManage.setHomeworkName(homeworkExamDTO.getName());
            homeworkManage.setHomeworkType(homeworkExamDTO.getType());
            homeworkManage.setHomeworkClass(className);
            homeworkManage.setPassRate(BigDecimal.valueOf(0));
            homeworkManage.setStartingTime(homeworkExamDTO.getStartingTime());
            homeworkManage.setEndTime(homeworkExamDTO.getEndTime());
            homeworkManage.setState(0);
            homeworkManage.setHomeworkAsk(homeworkExamDTO.getHomeworkAsk());

            String upload = localFileUtil.upload(file);

            homeworkManage.setHomeworkAnnex(upload);
            homeworkManage.setCreateTime(new Date());

            return homeworkManageMapper.insert(homeworkManage);

        }

        if(homeworkExamDTO.getType().equals("考试")){
            //考试信息赋值
            TchExamManage examManage=new TchExamManage();

            examManage.setExamId(string);
            examManage.setExamName(homeworkExamDTO.getName());
            examManage.setExamClass(className);
            examManage.setExamStartingTime(homeworkExamDTO.getStartingTime());
            examManage.setExamEndTime(homeworkExamDTO.getEndTime());
            examManage.setAverageAccuracy(BigDecimal.valueOf(0));
            examManage.setState(0);
            examManage.setUserId(homeworkExamDTO.getUserId());
            examManage.setCreateTime(new Date());
            examManage.setAnswersFrequency(homeworkExamDTO.getAnswersFrequency());
            examManage.setTopicNumberState(homeworkExamDTO.getTopicNumberState());
            examManage.setOptionNumberState(homeworkExamDTO.getOptionNumberState());
            examManage.setParsingState(homeworkExamDTO.getParsingState());
            examManage.setAnswerSate(homeworkExamDTO.getAnswerSate());
            examManage.setTopicId(topicId);

            insert = examManageMapper.insert(examManage);
        }

        //作业或考试添加成功，则继续添加选题详情

        if(insert>1){
            //选题详情赋值,取出选题详情集合
            List<TchTopicDetails> tchTopicDetails = convertUtil.DTOListToEntity(topicDetailsDTOS, TchTopicDetails.class);

            for (TchTopicDetails tchTopicDetail: tchTopicDetails) {
                tchTopicDetail.setId(UUID.randomUUID().toString());
                tchTopicDetail.setTaskName(homeworkExamDTO.getType());
                tchTopicDetail.setTaskId(string);

                return topicDetailsMapper.insert(tchTopicDetail);
            }
        }

        return 0;
    }

    @Override
    public Integer update(TchHomeworkExamDTO homeworkExamDTO, MultipartFile file) {

        //将班级集合和题目id集合转为字符串
        String className="";
        for (int i = 0; i < homeworkExamDTO.getClassNameList().size(); i++) {

            if(homeworkExamDTO.getClassNameList().size()-1==i){
                className=className+homeworkExamDTO.getClassNameList().get(0);
            }else {
                className=className+homeworkExamDTO.getClassNameList().get(0)+",";
            }
        }

        // TODO: 2024/8/6 取出所选题的数量，在题库中随机抽取，将题目ID拼接成字符串存到数据库中
        //取出所选题的总数量
        List<TchTopicDetailsDTO> topicDetailsDTOS = homeworkExamDTO.getTopicDetailsDTOS();
        int quantity=0;
        for (TchTopicDetailsDTO topicDetailsDTO: topicDetailsDTOS) {
            quantity=quantity+topicDetailsDTO.getQuantity();
        }

        //查询题库所有题
        List<StuLearningAssessment> stuLearningAssessments = learningAssessmentMapper.selectByExample(null);

        if(quantity>stuLearningAssessments.size()){
            throw new ServiceException(HttpStatus.ACCEPTED,"要选择的题目数量不能大于题库的总数量");
        }

        // 打乱集合
        Collections.shuffle(stuLearningAssessments);

        // 获取前quantity个元素
        List<StuLearningAssessment> subList = stuLearningAssessments.subList(0, quantity);

        //遍历集合，取出id拼接成字符串
        String topicId="";
        for (int i = 0; i < subList.size(); i++) {
            if(subList.size()-1==i){
                topicId=topicId+subList.get(0).getTopicId();
            }else {
                topicId=topicId+subList.get(0).getTopicId()+",";
            }
        }

        // TODO: 2024/8/6 1、题库作业，2、附件作业，3、考试
        int update=0; //用于判断作业或考试是否更新成功
        if(homeworkExamDTO.getType().equals("题库作业")){
            //查询到要更新的数据
            TchHomeworkManage tchHomeworkManage = homeworkManageMapper.selectByPrimaryKey(homeworkExamDTO.getId());

            //更新题库作业信息
            tchHomeworkManage.setHomeworkName(homeworkExamDTO.getName());
            tchHomeworkManage.setHomeworkClass(className);
            tchHomeworkManage.setStartingTime(homeworkExamDTO.getStartingTime());
            tchHomeworkManage.setEndTime(homeworkExamDTO.getEndTime());
            tchHomeworkManage.setUpdateTime(new Date());
            tchHomeworkManage.setAnswersFrequency(homeworkExamDTO.getAnswersFrequency());
            tchHomeworkManage.setTopicNumberState(homeworkExamDTO.getTopicNumberState());
            tchHomeworkManage.setOptionNumberState(homeworkExamDTO.getOptionNumberState());
            tchHomeworkManage.setParsingState(homeworkExamDTO.getParsingState());
            tchHomeworkManage.setAnswerSate(homeworkExamDTO.getAnswerSate());
            tchHomeworkManage.setTopicId(topicId);

            update=homeworkManageMapper.updateByPrimaryKeySelective(tchHomeworkManage);
        }

        if(homeworkExamDTO.getType().equals("附件作业")){
            //查询到要更新的数据
            TchHomeworkManage tchHomeworkManage = homeworkManageMapper.selectByPrimaryKey(homeworkExamDTO.getId());

            //更新附件作业信息

            tchHomeworkManage.setHomeworkName(homeworkExamDTO.getName());
            tchHomeworkManage.setHomeworkClass(className);
            tchHomeworkManage.setStartingTime(homeworkExamDTO.getStartingTime());
            tchHomeworkManage.setEndTime(homeworkExamDTO.getEndTime());
            tchHomeworkManage.setHomeworkAsk(homeworkExamDTO.getHomeworkAsk());

            String upload = localFileUtil.upload(file);

            tchHomeworkManage.setHomeworkAnnex(upload);
            tchHomeworkManage.setUpdateTime(new Date());

            return homeworkManageMapper.updateByPrimaryKeySelective(tchHomeworkManage);
        }

        if(homeworkExamDTO.getType().equals("考试")){
            TchExamManage examManage = examManageMapper.selectByPrimaryKey(homeworkExamDTO.getId());

            //更新考试信息
            examManage.setExamName(homeworkExamDTO.getName());
            examManage.setExamClass(className);
            examManage.setExamStartingTime(homeworkExamDTO.getStartingTime());
            examManage.setExamEndTime(homeworkExamDTO.getEndTime());
            examManage.setUserId(homeworkExamDTO.getUserId());
            examManage.setUpdateTime(new Date());
            examManage.setAnswersFrequency(homeworkExamDTO.getAnswersFrequency());
            examManage.setTopicNumberState(homeworkExamDTO.getTopicNumberState());
            examManage.setOptionNumberState(homeworkExamDTO.getOptionNumberState());
            examManage.setParsingState(homeworkExamDTO.getParsingState());
            examManage.setAnswerSate(homeworkExamDTO.getAnswerSate());
            examManage.setTopicId(topicId);

            update=examManageMapper.updateByPrimaryKeySelective(examManage);

        }

        //作业或考试更新成功，则继续更新选题详情
        if(update>0){

            //取出选题详情集合
            List<TchTopicDetails> tchTopicDetails = convertUtil.DTOListToEntity(topicDetailsDTOS, TchTopicDetails.class);

            //根据考试或作业ID查询选题详情信息
            TchTopicDetailsExample topicDetailsExample=new TchTopicDetailsExample();
            topicDetailsExample.createCriteria().andTaskIdEqualTo(homeworkExamDTO.getId());

            List<TchTopicDetails> tchTopicDetailsList = topicDetailsMapper.selectByExample(topicDetailsExample);

            for (int i = 0; i < tchTopicDetails.size(); i++) {
                for (int j = 0; j < tchTopicDetailsList.size(); j++) {

                    if(tchTopicDetails.get(i).getTopicType().equals(tchTopicDetailsList.get(j).getTopicType())){ //判断题型是否相等
                        //更新
                        tchTopicDetails.get(i).setId(tchTopicDetailsList.get(j).getId());

                        topicDetailsMapper.updateByPrimaryKeySelective(tchTopicDetails.get(i));
                        break;
                    }
                    //如果没有一样的题型，则该题型为新增题型，直接添加

                    tchTopicDetails.get(i).setId(UUID.randomUUID().toString());
                    tchTopicDetails.get(i).setTaskName(homeworkExamDTO.getType());
                    tchTopicDetails.get(i).setTaskId(homeworkExamDTO.getId());

                    topicDetailsMapper.insert(tchTopicDetails.get(i));

                }
            }
            return 1;

        }

        return 0;
    }

    @Override
    public List<String> getSelect(String userId) {
        TchStartCourseManageExample startCourseManageExample=new TchStartCourseManageExample();
        startCourseManageExample.createCriteria().andUserIdEqualTo(userId);

        List<TchStartCourseManage> startCourseManages = startCourseManageMapper.selectByExample(startCourseManageExample);

        List<String> list=new ArrayList<>();
        if(startCourseManages.isEmpty()){
            throw new ServiceException(HttpStatus.ACCEPTED,"未添加开课班级");
        }else {

            for (TchStartCourseManage startCourseManage: startCourseManages) {

                //取到开课班级名称
                String startCourseClassName = startCourseManage.getStartCourseClassName();
                String[] split = startCourseClassName.split(",");

                list.addAll(Arrays.asList(split));
            }
        }

        return list;
    }

    @Override
    public List<TchTopicTypeDTO> getTopicType() {
        return learningAssessmentMapper.getTopicType();
    }

    @Override
    public TchHomeworkExamDTO getById(String id,String type) {
        if(type.equals("题库作业")){
            //查询作业信息
            TchHomeworkManage tchHomeworkManage = homeworkManageMapper.selectByPrimaryKey(id);

            //转为DTO
            TchHomeworkExamDTO tchHomeworkExamDTO = convertUtil.entityToDTO(tchHomeworkManage, TchHomeworkExamDTO.class);

            //查询选题详情信息
            TchTopicDetailsExample topicDetailsExample=new TchTopicDetailsExample();
            topicDetailsExample.createCriteria().andTaskIdEqualTo(id);
            List<TchTopicDetails> tchTopicDetails = topicDetailsMapper.selectByExample(topicDetailsExample);

            //转为DTO
            List<TchTopicDetailsDTO> tchTopicDetailsDTOS = convertUtil.entityToDTOList(tchTopicDetails, TchTopicDetailsDTO.class);

            //赋值
            tchHomeworkExamDTO.setId(tchHomeworkManage.getHomeworkId());
            tchHomeworkExamDTO.setName(tchHomeworkManage.getHomeworkName());
            tchHomeworkExamDTO.setType(tchHomeworkManage.getHomeworkType());
            //班级集合
            String[] split = tchHomeworkManage.getHomeworkClass().split(",");
            List<String> list=new ArrayList<>(Arrays.asList(split));
            tchHomeworkExamDTO.setClassNameList(list);
            tchHomeworkExamDTO.setTopicDetailsDTOS(tchTopicDetailsDTOS);

            return tchHomeworkExamDTO;
        }

        if(type.equals("附件作业")){
            //查询作业信息
            TchHomeworkManage tchHomeworkManage = homeworkManageMapper.selectByPrimaryKey(id);

            //转为DTO
            TchHomeworkExamDTO tchHomeworkExamDTO = convertUtil.entityToDTO(tchHomeworkManage, TchHomeworkExamDTO.class);

            //赋值
            tchHomeworkExamDTO.setId(tchHomeworkManage.getHomeworkId());
            tchHomeworkExamDTO.setName(tchHomeworkManage.getHomeworkName());
            tchHomeworkExamDTO.setType(tchHomeworkManage.getHomeworkType());
            //班级集合
            String[] split = tchHomeworkManage.getHomeworkClass().split(",");
            List<String> list=new ArrayList<>(Arrays.asList(split));
            tchHomeworkExamDTO.setClassNameList(list);

            return tchHomeworkExamDTO;
        }

        if(type.equals("考试")){
            //查询考试信息
            TchExamManage examManage = examManageMapper.selectByPrimaryKey(id);

            //查询选题详情信息
            TchTopicDetailsExample topicDetailsExample=new TchTopicDetailsExample();
            topicDetailsExample.createCriteria().andTaskIdEqualTo(id);
            List<TchTopicDetails> tchTopicDetails = topicDetailsMapper.selectByExample(topicDetailsExample);

            //转为DTO
            List<TchTopicDetailsDTO> tchTopicDetailsDTOS = convertUtil.entityToDTOList(tchTopicDetails, TchTopicDetailsDTO.class);

            //赋值
            TchHomeworkExamDTO dto=new TchHomeworkExamDTO();
            dto.setId(examManage.getExamId());
            dto.setUserId(examManage.getUserId());
            dto.setName(examManage.getExamName());
            dto.setType(type);
            //班级集合
            String[] split = examManage.getExamClass().split(",");
            List<String> list=new ArrayList<>(Arrays.asList(split));
            dto.setClassNameList(list);
            dto.setStartingTime(examManage.getExamStartingTime());
            dto.setEndTime(examManage.getExamEndTime());
            dto.setTopicDetailsDTOS(tchTopicDetailsDTOS);

            return dto;

        }

        return null;
    }
}
